GuildPlannerPro_Skills = {}

function GuildPlannerPro_Skills:MapChampionPoints()
    local champion = {
        Disciplines = {},
    }
    if IsChampionSystemUnlocked() then
        local numChampionDiscliplines = GetNumChampionDisciplines()
        for disciplineIndex = 1, numChampionDiscliplines do
            local disciplineId = GetChampionDisciplineId(disciplineIndex)
            local skills = {}
            local numSkills = GetNumChampionDisciplineSkills(disciplineIndex)
            for skillIndex = 1, numSkills do
                local skillId = GetChampionSkillId(disciplineIndex, skillIndex)
                local normalizedX, normalizedY = GetChampionSkillPosition(skillId)
                local normalizedOffsetX, normalizedOffsetY = GetChampionClusterRootOffset(skillId)
                local maxPoints = GetChampionSkillMaxPoints(skillId)
                local jumpPoints = { GetChampionSkillJumpPoints(skillId) }

                local bonusSteps, prevBonusText, currBonusText = nil, '', ''
                if #jumpPoints > 2 then
                    bonusSteps = {}
                    for bonusPoint = 1, maxPoints do
                        currBonusText = GetChampionSkillCurrentBonusText(skillId, bonusPoint)
                        if currBonusText and currBonusText ~= prevBonusText then
                            bonusSteps[bonusPoint] = currBonusText
                        end
                        prevBonusText = currBonusText
                    end
                end

                local nodeUnlocksAtXPoints = 0
                for bonusPoint = 1, maxPoints do
                    local wouldNodeBeUnlocked = WouldChampionSkillNodeBeUnlocked(skillId, bonusPoint)
                    if wouldNodeBeUnlocked and nodeUnlocksAtXPoints == 0 then
                        nodeUnlocksAtXPoints = bonusPoint
                    end
                end

                skills[skillIndex] = {
                    Id = skillId,
                    Name = GetChampionSkillName(skillId),
                    Description = GetChampionSkillDescription(skillId, 0),
                    Type = GetChampionSkillType(skillId),
                    MaxPoints = maxPoints,
                    NodeUnlocksAtXPoints = nodeUnlocksAtXPoints,
                    BonusSteps = bonusSteps,
                    ChampionAbilityId = GetChampionAbilityId(skillId),
                    LinkedSkillIds = { GetChampionSkillLinkIds(skillId) },
                    JumpPoints = jumpPoints,
                    IsRootNode = IsChampionSkillRootNode(skillId),
                    IsClusterRoot = IsChampionSkillClusterRoot(skillId),
                    ClusterName = GetChampionClusterName(skillId),
                    ClusterBackground = GetChampionClusterBackgroundTexture(skillId),
                    ClusterSkillIds = { GetChampionClusterSkillIds(skillId) },
                    PositionX = normalizedX,
                    PositionY = normalizedY,
                    ClusterRootOffsetX = normalizedOffsetX,
                    ClusterRootOffsetY = normalizedOffsetY,
                }
            end

            champion["Disciplines"][disciplineIndex] = {
                Id = disciplineId,
                Name = GetChampionDisciplineName(disciplineId),
                Type = GetChampionDisciplineType(disciplineId),
                ZoomedOutBg = GetChampionDisciplineZoomedOutBackground(disciplineId),
                ZoomedInBg = GetChampionDisciplineZoomedInBackground(disciplineId),
                Skills = skills,
            }
        end
    end

    return champion
end

function GuildPlannerPro_Skills:ExportChampionPoints()
    local champion = {
        Disciplines = {},
    }
    if IsChampionSystemUnlocked() then
        local numChampionDiscliplines = GetNumChampionDisciplines()
        for disciplineIndex = 1, numChampionDiscliplines do
            local disciplineId = GetChampionDisciplineId(disciplineIndex)
            local skills = {}
            local numSkills = GetNumChampionDisciplineSkills(disciplineIndex)
            for skillIndex = 1, numSkills do
                local skillId = GetChampionSkillId(disciplineIndex, skillIndex)
                local pointsSpent = GetNumPointsSpentOnChampionSkill(skillId)
                table.insert(skills, {
                    SkillId = skillId,
                    SkillIndex = skillIndex,
                    CurrentBonusText = GetChampionSkillCurrentBonusText(skillId, pointsSpent),
                    PointsSpent = pointsSpent,
                    MaxPossiblePoints = GetMaxPossiblePointsInChampionSkill(skillIndex),
                })
            end

            table.insert(champion["Disciplines"], {
                DisciplineId = disciplineId,
                DisciplineIndex = disciplineIndex,
                PointsSpent = GetNumSpentChampionPoints(disciplineId),
                PointsUnspent = GetNumUnspentChampionPoints(disciplineId),
                Skills = skills,
            })
        end
    end

    return champion
end

function GuildPlannerPro_Skills:MapSkillLines()
    local skillLines = {}
    if not AreSkillsInitialized() then
        GuildPlannerPro_Utils:PrintMessage("Skills not initialized yet.")

        return
    end

    local classSkillLineMap = {
        [1] = { 35, 36, 37 }, -- Dragonknight
        [2] = { 41, 42, 43 }, -- Sorcerer
        [3] = { 38, 39, 40 }, -- Nightblade
        [4] = { 22, 27, 28 }, -- Templar
        [5] = { 127, 128, 129 }, -- Warden
        [6] = { 131, 132, 133 }, -- Necromancer
        [7] = { 218, 219, 220 }, -- Arcanist
    }

    local dataMap = SKILLS_DATA_MANAGER.abilityIdToProgressionDataMap
    for _, skill in pairs(dataMap) do
        local abilityId = skill.abilityId

        if not skillLines[abilityId] then
            skillLines[abilityId] = {}

            local tankAbility, healerAbility, ddAbility = GetAbilityRoles(abilityId)
            skillLines[abilityId].Roles = {
                IsTankAbility = tankAbility,
                IsHealerAbility = healerAbility,
                IsDdAbility = ddAbility,
            }

            local skillData = skill.skillData
            skillLines[abilityId].Id = abilityId
            skillLines[abilityId].Name = skill.name
            skillLines[abilityId].Icon = skill.icon
            skillLines[abilityId].IsUltimate = skillData:IsUltimate()
            skillLines[abilityId].IsPassive = skillData:IsPassive()
            skillLines[abilityId].ProgressionKey = skill.skillProgressionKey
            skillLines[abilityId].DescriptionHeader = GetAbilityDescriptionHeader(abilityId)
            if not skillData:IsPassive() then
                skillLines[abilityId].FormattedName = skill.name .. ' IV'
                local channeled, castTime, channelTime = GetAbilityCastInfo(abilityId, MAX_RANKS_PER_ABILITY)
                local mechanic = GetNextAbilityMechanicFlag(abilityId)
                local cost = GetAbilityCost(abilityId, mechanic, MAX_RANKS_PER_ABILITY)
                local costOverTime, chargeFreqMs = GetAbilityCostOverTime(abilityId, mechanic, MAX_RANKS_PER_ABILITY)
                skillLines[abilityId].IsChanneled = channeled
                skillLines[abilityId].ChannelTime = channelTime
                skillLines[abilityId].CastTime = castTime
                skillLines[abilityId].Radius = GetAbilityRadius(abilityId, MAX_RANKS_PER_ABILITY)
                local _, maxRangeCm = GetAbilityRange(abilityId, MAX_RANKS_PER_ABILITY)
                skillLines[abilityId].Range = maxRangeCm
                skillLines[abilityId].AngleDistance = GetAbilityAngleDistance(abilityId)
                skillLines[abilityId].DurationToggled = IsAbilityDurationToggled(abilityId)
                skillLines[abilityId].DurationMs = GetAbilityDuration(abilityId, MAX_RANKS_PER_ABILITY)
                skillLines[abilityId].Cooldown = GetAbilityCooldown(abilityId)
                skillLines[abilityId].Target = GetAbilityTargetDescription(abilityId, MAX_RANKS_PER_ABILITY)
                skillLines[abilityId].Cost = {
                    Cost = cost,
                    Mechanic = GuildPlannerPro_Const.CombatMechanicFlags[mechanic],
                }
                skillLines[abilityId].CostOverTime = {
                    Cost = costOverTime,
                    Mechanic = GuildPlannerPro_Const.CombatMechanicFlags[mechanic],
                    ChargeFreqMs = chargeFreqMs,
                }
                skillLines[abilityId].IsBase = skill:IsBase()
                skillLines[abilityId].IsMorph = skill:IsMorph()
                skillLines[abilityId].DescriptionBody = GetAbilityDescription(abilityId, MAX_RANKS_PER_ABILITY, "player")
                if skillLines[abilityId].IsMorph then
                    skillLines[abilityId].MorphSlot = skill:GetMorphSlot()
                    local siblingMorphData = skill:GetSiblingMorphData()
                    skillLines[abilityId].SiblingMorphAbilityId = siblingMorphData.abilityId
                end
            else
                skillLines[abilityId].FormattedName = skill:GetFormattedNameWithRank()
                skillLines[abilityId].LineRankNeededToUnlock = skill:GetLineRankNeededToUnlock()
                skillLines[abilityId].DescriptionBody = GetAbilityDescription(abilityId, skill:GetRank(), "")
            end

            skillLines[abilityId].LineRankNeededToPurchase = skillData.lineRankNeededToPurchase
            skillLines[abilityId].SkillIndex = skillData.skillIndex
            skillLines[abilityId].HeaderText = skillData:GetHeaderText()

            local skillLineData = skillData.skillLineData
            skillLines[abilityId].SkillLineId = skillLineData.id
            skillLines[abilityId].SkillLineName = skillLineData:GetName()
            skillLines[abilityId].SkillLineIndex = skillLineData:GetSkillLineIndex()
            skillLines[abilityId].SkillLineUnlockText = skillLineData:GetUnlockText()
            skillLines[abilityId].SkillLineIsWerewolf = skillLineData:IsWerewolf()

            local skillTypeData = skillLineData.skillTypeData
            skillLines[abilityId].SkillType = skillTypeData.skillType
            skillLines[abilityId].SkillTypeName = skillTypeData.name
            if skillLines[abilityId].SkillType == SKILL_TYPE_CLASS then
                for classIndex, classSkillLines in pairs(classSkillLineMap) do
                    if GuildPlannerPro_Utils:InTable(skillLines[abilityId].SkillLineId, classSkillLines) then
                        skillLines[abilityId].SkillTypeClassIndex = classIndex
                        skillLines[abilityId].SkillTypeClassId = GetClassInfo(classIndex)
                        break
                    end
                end
            end

            local progressionData = skillData.skillProgressions

            skillLines[abilityId].NewEffectLine = GetAbilityNewEffectLines(abilityId)
            skillLines[abilityId].UpgradeLines = GetAbilityUpgradeLines(abilityId)

            for key, morphData in pairs(progressionData) do
                if skillLines[abilityId].IsPassive then
                    skillLines[abilityId].ProgressionMap = skillLines[abilityId].ProgressionMap or {}
                    table.insert(skillLines[abilityId].ProgressionMap, {
                        ProgressionKey = key,
                        ProgressionAbilityId = morphData.abilityId,
                    })
                else
                    skillLines[abilityId].Morphs = skillLines[abilityId].Morphs or {}
                    table.insert(skillLines[abilityId].Morphs, {
                        ProgressionKey = key,
                        ProgressionAbilityId = morphData.abilityId,
                    })
                end
            end
        end
    end

    return skillLines
end

function GuildPlannerPro_Skills:ExportCharacterSkillLines()
    local skillLines = {}
    local abilityIdToProgressionDataMap = SKILLS_DATA_MANAGER.abilityIdToProgressionDataMap
    for _, skill in pairs(abilityIdToProgressionDataMap) do
        local abilityId = skill.abilityId
        local skillData = skill.skillData
        local skillLineData = skillData.skillLineData
        local progressionData = skillData.skillProgressions
        if skillLineData:IsPlayerSkillLine() and skillData.isPurchased then
            if not skillLines[abilityId] then
                local currentSkillProgressionKey = skillData:GetCurrentSkillProgressionKey()
                if not skillLines[progressionData[currentSkillProgressionKey].abilityId] then
                    skillLines[progressionData[currentSkillProgressionKey].abilityId] = {}
                end
            end

            if skillLines[abilityId] then
                skillLines[abilityId].Id = abilityId
                skillLines[abilityId].DetailedName = skill:GetDetailedName()
                skillLines[abilityId].FormattedName = skill:GetFormattedName()
            end
        end
    end

    return skillLines
end

function GuildPlannerPro_Skills:ExportCharacterHotBars()
    local hotbars = {}

    local hotBarsToExport = { HOTBAR_CATEGORY_PRIMARY, HOTBAR_CATEGORY_BACKUP, HOTBAR_CATEGORY_CHAMPION }
    if GetPlayerCurseType() == CURSE_TYPE_WEREWOLF then
        table.insert(hotBarsToExport, HOTBAR_CATEGORY_WEREWOLF)
    end

    local abilityIdToProgressionDataMap = SKILLS_DATA_MANAGER.abilityIdToProgressionDataMap
    for _, hotBarCategory in ipairs(hotBarsToExport) do
        hotbars[hotBarCategory] = {}
        local startActionSlotIndex, endActionSlotIndex
        if hotBarCategory ~= HOTBAR_CATEGORY_CHAMPION then
            startActionSlotIndex, endActionSlotIndex = GetAssignableAbilityBarStartAndEndSlots()
        else
            startActionSlotIndex, endActionSlotIndex = GetAssignableChampionBarStartAndEndSlots()
        end
        for slotIndex = startActionSlotIndex, endActionSlotIndex do
            if IsSlotUsed(slotIndex, hotBarCategory) then
                local slotType = GetSlotType(slotIndex, hotBarCategory)
                hotbars[hotBarCategory][slotIndex] = {
                    Type = slotType,
                }
                local boundId = GetSlotBoundId(slotIndex, hotBarCategory)
                if slotType == ACTION_TYPE_ABILITY then
                    local skill = abilityIdToProgressionDataMap[boundId]
                    hotbars[hotBarCategory][slotIndex]["BoundId"] = skill.abilityId
                    hotbars[hotBarCategory][slotIndex]["Name"] = skill:GetDetailedName()
                elseif slotType == ACTION_TYPE_CHAMPION_SKILL then
                    hotbars[hotBarCategory][slotIndex]["BoundId"] = boundId
                    hotbars[hotBarCategory][slotIndex]["Name"] = GetChampionSkillName(boundId)
                end
            end
        end
    end

    return hotbars
end
